# Copyright (c) 2012 Cloudera, Inc. All rights reserved.

try:
  from cStringIO import StringIO
except:
  from StringIO import StringIO

import time

class HostDirUpdate(object):
  """
  This models the HostDirUpdate Avro record and gives it a bit more structure.
  """
  def __init__(self, path, mountpoint, timestamp=None):
    if timestamp is None:
      timestamp = time.time()
    self._timestamp = timestamp

    self._path = path
    self._mount = mountpoint
    self._metrics = [ ]

  def add_metric(self, metricid, value):
    self._metrics.append((metricid, value))
    return self

  def __str__(self):
    sio = StringIO()
    sio.write(">> %s:" %
              time.strftime("%x %X", time.localtime(self._timestamp)))
    sio.write(">>  %s (on %s)" % (self._path, self._mount))
    for metricid, val in self._metrics:
      sio.write("\n   %-10d : %s" % (metricid, val))
    return sio.getvalue()

  def to_avro_dict(self):
    """Return a Python dictionary that maps to the Avro struct"""
    return dict(path=self._path,
                mountpoint=self._mount,
                metrics=[ dict(id=k, value=v) for (k, v) in self._metrics ])
