#!/bin/bash

# Copyright (c) 2011-2012 Cloudera, Inc. All rights reserved.

# Time marker for both stderr and stdout
date; date 1>&2

cloudera_config=`dirname $0`
cloudera_config=`cd "$cloudera_config/../common"; pwd`
. ${cloudera_config}/cloudera-config.sh

# Attempt to find Java and place it in JAVA_HOME. The catalina.sh script expects
# to find it there.
locate_java_home

# This will be used later while upgrading Oozie database
OLD_OOZIE_HOME=$CDH_OOZIE_HOME

# Load the parcel environment
source_parcel_environment

set -x

function check_binary_exists()
{
  which $1
  if [[ $? -ne 0 ]]; then
    echo "$1 utility not found. It is required for running Oozie."
    exit 1
  fi
}

echo "using $CDH_VERSION as CDH_VERSION"

# This horrifying mess of variables represents an amalgam of the actions
# performed by oozie-run.sh and other Oozie helper scripts. We can't use these
# scripts directly because SCM disagrees with too many of their assumptions
# (e.g. the Oozie configuration is in /etc).
#
# SCM is expected to set the following:
# - oozie_log_dir
# - oozie_log_file
# - oozie_http_hostname
# - oozie_http_port
# - oozie_admin_port (CDH4 only)
# - oozie_web_console
# - OOZIE_SERVER_OPTS
# - OOZIE_SERVER_DIR (CDH4 only)

export OOZIE_HOME=$CDH_OOZIE_HOME
export OOZIE_CONFIG=$PWD
export OOZIE_LOG=$oozie_log_dir
export OOZIE_LOG_FILE=$oozie_log_file
export OOZIE_CONFIG_FILE=oozie-site.xml
export OOZIE_LOG4J_FILE=log4j.properties
export OOZIE_LOG4J_RELOAD=10
export OOZIE_HTTP_HOSTNAME=$oozie_http_hostname
export OOZIE_HTTP_PORT=$oozie_http_port
if [ "3" = "$CDH_VERSION" ]; then
  export CATALINA_BASE=/var/lib/oozie/oozie-server
elif [ "4" = "$CDH_VERSION" ]; then
  export CATALINA_BASE=$OOZIE_HOME/$OOZIE_SERVER_DIR
  export OOZIE_ADMIN_PORT=$oozie_admin_port
fi

# local replacement of substitution variables.
# i.e. master doesn't know final resting place of configuration files
#   which may require absolute path (e.g. dfs_allows)
if [[ -d $CONF_DIR/yarn-conf ]]; then
  export HADOOP_CONF_DIR=$CONF_DIR/yarn-conf
  export HADOOP_MAPRED_HOME=$CDH_MR2_HOME
  perl -pi -e "s#{{CDH_MAPRED_HOME}}#$CDH_MR2_HOME#g" $HADOOP_CONF_DIR/*
  perl -pi -e "s#{{CDH_HADOOP_HOME}}#$CDH_HADOOP_HOME#g" $HADOOP_CONF_DIR/*
fi

if [ "install-sharelib" = "$1" ]; then
  acquire_kerberos_tgt oozie.keytab
  HDFS_CFG=$OOZIE_CONFIG/hadoop-conf
  OOZIE_DIR=/user/`whoami`
  hadoop --config $HDFS_CFG fs -rmr $OOZIE_DIR/share

  # Unpack the sharelibs into the staging directory.
  UNPACKED_DIR=$(mktemp -d -t oozie-sharelib-staging.XXXXXXXXXX)
  tar -C $UNPACKED_DIR -xzf $OOZIE_HOME/$2 #$2 is the name of the tar file to put in HDFS
  hadoop --config $HDFS_CFG fs -put $UNPACKED_DIR/share share
  exit $?
fi

echo "using $CATALINA_BASE as CATALINA_BASE"
if [ ! -d $CATALINA_BASE ]; then
  echo "$CATALINA_BASE doesn't exist"
  exit 1
fi

export CATALINA_TMPDIR=$PWD/temp
export CATALINA_OUT=$OOZIE_LOG
export CATALINA_PID=$PWD/oozie.pid

OPTIONS="$OOZIE_SERVER_OPTS"
OPTIONS="$OPTIONS -Doozie.home.dir=${OOZIE_HOME}"
OPTIONS="$OPTIONS -Doozie.config.dir=$OOZIE_CONFIG"
OPTIONS="$OPTIONS -Doozie.log.dir=$OOZIE_LOG"
OPTIONS="$OPTIONS -Doozie.log.file=$OOZIE_LOG_FILE"
OPTIONS="$OPTIONS -Doozie.config.file=$OOZIE_CONFIG_FILE"
OPTIONS="$OPTIONS -Doozie.log4j.file=$OOZIE_LOG4J_FILE"
OPTIONS="$OPTIONS -Doozie.log4j.reload=$OOZIE_LOG4J_RELOAD"
OPTIONS="$OPTIONS -Doozie.http.hostname=$OOZIE_HTTP_HOSTNAME"
OPTIONS="$OPTIONS -Doozie.http.port=$OOZIE_HTTP_PORT"
OPTIONS="$OPTIONS -Djava.net.preferIPv4Stack=true" # Disable IPv6.
if [ "3" != "$CDH_VERSION" ]; then
  OPTIONS="$OPTIONS -Doozie.admin.port=$OOZIE_ADMIN_PORT"
fi
OPTIONS="$OPTIONS -Dderby.stream.error.file=$OOZIE_LOG/derby.log"
export CATALINA_OPTS="$OPTIONS $CATALINA_OPTS"

if [ "db-command" = "$1" ]; then
  if [ "4" != "$CDH_VERSION" ]; then
    echo "Oozie in CDH_VERSION = $CDH_VERSION does not support db-commands"
    exit 1
  fi

  DBCOMMAND=$2
  if [ "" = "$DBCOMMAND" ]; then
    echo "db-command needs an additional argument."
    exit 1
  fi

  
  # If we are using parcels, we try to copy any old jdbc jars to the new location.
  # $3 is the name of the jar to copy
  if [ "upgrade" = "$DBCOMMAND" ] && [ "$OLD_OOZIE_HOME" != "$CDH_OOZIE_HOME" ] && [ "" != "$3" ]; then
    if [ -e $OLD_OOZIE_HOME/libext/$3 ] && [ ! -e $CDH_OOZIE_HOME/libext/$3 ]; then
      cp $OLD_OOZIE_HOME/libext/$3 $CDH_OOZIE_HOME/libext
    fi
  fi

  OOZIECPPATH=""
  if [[ -d $OOZIE_HOME/libtools ]]; then
    for i in "${OOZIE_HOME}/libtools/"*.jar; do
      OOZIECPPATH="${OOZIECPPATH}:$i"
    done
  fi
  if [[ -d $OOZIE_HOME/libext ]]; then
    for i in "${OOZIE_HOME}/libext/"*.jar; do
      OOZIECPPATH="${OOZIECPPATH}:$i"
    done
  fi

  exec $JAVA_HOME/bin/java ${OPTIONS} -cp ${OOZIECPPATH} org.apache.oozie.tools.OozieDBCLI $DBCOMMAND -run
fi

# Create temp directory for Catalina
mkdir -p $CATALINA_TMPDIR

# CDH4 Oozie can be run directly
if [ "4" = "$CDH_VERSION" ]; then
  exec $TOMCAT_HOME/bin/catalina.sh run
fi

# CDH3 Oozie requires re-packing the war
# to add in ExtJS and MySQL as necessary.
SOURCE_WAR=$OOZIE_HOME/oozie.war
DEST_WAR=$CATALINA_BASE/webapps/oozie.war

# Look for ExtJS and/or additional JAR files in the Oozie extension directory,
# and if found, add them to the Oozie WAR file.
#
# TODO: Move this into a separate command and/or do this only when we expect
# it to have a meaningful effect.
OOZIE_EXTENSION_DIR=$OOZIE_HOME/libext
if [[ -d $OOZIE_EXTENSION_DIR ]]; then

  # Check for unzip and zip before proceeding.
  check_binary_exists zip
  check_binary_exists unzip

  # Unpack the source WAR file into the staging directory.
  UNPACKED_WAR_DIR=$(mktemp -d -t oozie-war-staging.XXXXXXXXXX)
  unzip -o -q $SOURCE_WAR -d $UNPACKED_WAR_DIR

  # If the web console is to be enabled, unpack the ExtJS ZIP file into the new
  # WAR staging directory.
  if [[ $oozie_web_console == true ]]; then
    EXTJS=$OOZIE_EXTENSION_DIR/ext-2.2.zip
    if [[ ! -f $EXTJS ]]; then
      echo "$EXTJS not found. This file is required for the Oozie web console."
      exit 1
    fi
    unzip -o -q $EXTJS -d $UNPACKED_WAR_DIR
    if [[ ! -d $UNPACKED_WAR_DIR/ext-2.2 ]]; then
      echo "ExtJS 2.2 archive did not contain an ext-2.2 subdirectory. This is "
      echo "the directory structure required by the Oozie web console."
      exit 1
    fi
  fi

  # Copy additional JAR files into the new WAR staging directory.
  JAR_DEST_DIR=$UNPACKED_WAR_DIR/WEB-INF/lib
  JARS="$OOZIE_EXTENSION_DIR/*.jar"
  ls $JARS > /dev/null 2>&1
  if [[ $? -eq 0 ]]; then
    cp $JARS $JAR_DEST_DIR
  fi

  # And look for the MySQL connector JAR in our standard location.
  if [[ -e "$CLOUDERA_MYSQL_CONNECTOR_JAR" ]]; then
    cp "$CLOUDERA_MYSQL_CONNECTOR_JAR" $JAR_DEST_DIR
  fi

  # Delete the old WAR file because "zip" will append into it. 
  # Create the new WAR file.
  (cd $UNPACKED_WAR_DIR && rm -f $DEST_WAR && zip -q -r $DEST_WAR *)
  rm -rf $UNPACKED_WAR_DIR
else
  # No unpacking necessary, just copy the WAR file into place.
  cp $SOURCE_WAR $DEST_WAR
fi

# Delete the old unpacked WAR if it exists.
rm -rf $CATALINA_BASE/webapps/oozie

exec $OOZIE_HOME/oozie-server/bin/catalina.sh run
