// Copyright (c) 2012 Cloudera, Inc.  All rights reserved.
define([
  "cloudera/headlamp/hdfs/quota/Quota",
  "cloudera/common/I18n",
  "knockout"
], function(Quota, I18n, ko) {

  /**
   * Controls the Edit Quota Dialog.
   */
  return function(options) {
    var mode, viewModel = new Quota();

    var getDialog = function() {
      return $("#quotaEdit");
    };

    var openDialog = function(addOrEdit) {
      getDialog().dialog("open");
    };

    var closeDialog = function() {
      getDialog().dialog("close");
    };

    var getTable = function() {
      return $("#" + options.tableId);
    };

    /**
     * A FileSearchResult data is rendered (hidden) in the row.
     */
    var getDataFromRow = function($row) {
      var dataStr = $row.find(".data").text();
      return $.parseJSON(dataStr);
    };

    var updateTableRow = function(quotaJson) {
      try {
        var $table = getTable();
        var $lastQuotaRow = $.data($table[0], "lastQuotaRow");
        var $oTable = $table.dataTable();

        // Get the position of the current data from the node
        var aPos = $oTable.fnGetPosition( $lastQuotaRow[0] );
        // Get the data array for this row
        var aData = $oTable.fnGetData( aPos );

        // need the current index'ed values, merge with the old JSON
        // data.
        aData = $.merge(aData, getDataFromRow($lastQuotaRow));

        // update.
        aData.diskspaceQuota = quotaJson.dsLimit;
        aData.namespaceQuota = quotaJson.nsLimit;
        $oTable.fnUpdate(aData, aPos, 0);
      } catch (ex) {
        console.log(ex);
      }
    };

    var onOKClicked = function() {
      // Need to blur the input field, or else knockout won't
      // see the change.
      getDialog().parent().find("button:eq(0)").focus();
      var quotaJson = viewModel.toJSON();
      var onSetSuccess = function(response) {
        if (response === "OK") {
          updateTableRow(quotaJson);
        } else {
          // TODO: Fix this.
          alert(response);
        }
        closeDialog();
      };

      if (getDialog().valid()) {
        var urlParams = {
          data : JSON.stringify(quotaJson)
        };
        $.post(options.setQuotaUrl, urlParams, onSetSuccess);
      }
    };

    var onKeypressedInSearchFilter = function(evt) {
      var code = evt.keyCode || evt.which;
      // if user pressed enter, triggers the search button.
      if ($.ui.keyCode.ENTER === code) {
        onOKClicked();
      }
    };

    var onCancelClicked = function(evt) {
      closeDialog();
    };

    getDialog().dialog({
      autoOpen: false,
      title: I18n.t("ui.manageQuota"),
      minWidth: 480,
      minHeight: 320,
      modal: true,
      buttons: [{
        text: I18n.t("ui.ok"),
        click: onOKClicked
      }, {
        text: I18n.t("ui.cancel"),
        click: onCancelClicked
      }]
    });

    /**
     * Handles when the table is clicked.
     */
    getTable().click(function(evt) {
      var $target = $(evt.target);
      if ($target.is(".manageQuota") || $target.find(".manageQuota").length > 0) {
        viewModel.readValue(getDataFromRow($target.closest("tr")));
        openDialog();
        if (evt) {
          evt.preventDefault();
        }
        $.data($(this)[0], "lastQuotaRow", $target.closest("tr"));
      }
    });

    try {
      getDialog()
        .keypress(onKeypressedInSearchFilter)
        .validate({
          rules: {
            nsLimit: {
              min: 1
            },
            dsLimit: {
              min: 1
            }
          }
        });
    } catch (ex1) {
      console.log(ex1);
    }

    try {
      ko.applyBindings(viewModel, getDialog()[0]);
    } catch (ex2) {
      console.log(ex2);
    }
  };
});
